@@ -32,9 +32,10 @@ from collections import defaultdict
 from heapq import heappush, heapreplace
 
 from whoosh import analysis, fields, highlight, query, scoring
+from whoosh.automata import fst
 from whoosh.compat import xrange, string_type
-from whoosh.support import dawg
 from whoosh.support.levenshtein import distance
+from whoosh.util.text import utf8encode
 
 
 # Corrector objects
@@ -52,7 +53,7 @@ class Corrector(object):
             enough terms in the field within ``maxdist`` of the given word, the
             returned list will be shorter than this number.
         :param maxdist: the largest edit distance from the given word to look
-            at. Numbers higher than 2 are not very effective or efficient.
+            at. Values higher than 2 are not very effective or efficient.
         :param prefix: require suggestions to share a prefix of this length
             with the given word. This is often justifiable since most
             misspellings do not involve the first letter of the word. Using a
@@ -65,7 +66,11 @@ class Corrector(object):
         heap = []
         seen = set([text])
         for k in xrange(1, maxdist + 1):
-            for item in _suggestions(text, k, prefix, seen):
+            for item in _suggestions(text, k, prefix):
+                if item[1] in seen:
+                    continue
+                seen.add(item[1])
+
                 # Note that the *higher* scores (item[0]) are better!
                 if len(heap) < limit:
                     heappush(heap, item)
@@ -80,15 +85,14 @@ class Corrector(object):
         sugs = sorted(heap, key=lambda item: (0 - item[0], item[1]))
         return [sug for _, sug in sugs]
 
-    def _suggestions(self, text, maxdist, prefix, seen):
+    def _suggestions(self, text, maxdist, prefix):
         """Low-level method that yields a series of (score, "suggestion")
         tuples.
-        
+
         :param text: the text to check.
         :param maxdist: the maximum edit distance.
         :param prefix: require suggestions to share a prefix of this length
             with the given word.
-        :param seen: a set object with which to track already-seen words.
         """
 
         raise NotImplementedError
@@ -96,7 +100,7 @@ class Corrector(object):
 
 class ReaderCorrector(Corrector):
     """Suggests corrections based on the content of a field in a reader.
-    
+
     Ranks suggestions by the edit distance, then by highest to lowest
     frequency.
     """
@@ -105,56 +109,33 @@ class ReaderCorrector(Corrector):
         self.reader = reader
         self.fieldname = fieldname
 
-    def _suggestions(self, text, maxdist, prefix, seen):
+    def _suggestions(self, text, maxdist, prefix):
         fieldname = self.fieldname
         freq = self.reader.frequency
         for sug in self.reader.terms_within(fieldname, text, maxdist,
-                                            prefix=prefix, seen=seen):
+                                            prefix=prefix):
             # Higher scores are better, so negate the distance and frequency
-            score = 0 - (maxdist + (1.0 / freq(fieldname, sug) * 0.5))
+            # TODO: store spelling frequencies in the graph
+            f = freq(fieldname, sug) or 1
+            score = 0 - (maxdist + (1.0 / f * 0.5))
             yield (score, sug)
 
 
 class GraphCorrector(Corrector):
-    """Suggests corrections based on the content of a word list.
-    
+    """Suggests corrections based on the content of a raw
+    :class:`whoosh.automata.fst.GraphReader` object.
+
     By default ranks suggestions based on the edit distance.
     """
 
-    def __init__(self, word_graph):
-        self.word_graph = word_graph
+    def __init__(self, graph):
+        self.graph = graph
 
-    def _suggestions(self, text, maxdist, prefix, seen):
-        for sug in dawg.within(self.word_graph, text, maxdist, prefix=prefix,
-                               seen=seen):
+    def _suggestions(self, text, maxdist, prefix):
+        for sug in fst.within(self.graph, text, k=maxdist, prefix=prefix):
             # Higher scores are better, so negate the edit distance
             yield (0 - maxdist, sug)
 
-    def to_file(self, f):
-        """
-        
-        This method closes the file when it's done.
-        """
-
-        root = self.word_graph
-        dawg.DawgBuilder.reduce(root)
-        dawg.DawgWriter(f).write(root)
-
-    @classmethod
-    def from_word_list(cls, wordlist, strip=True):
-        dw = dawg.DawgBuilder(reduced=False)
-        for word in wordlist:
-            if strip:
-                word = word.strip()
-            dw.insert(word)
-        dw.finish()
-        return cls(dw.root)
-
-    @classmethod
-    def from_graph_file(cls, dbfile):
-        dr = dawg.DiskNode.load(dbfile)
-        return cls(dr)
-
 
 class MultiCorrector(Corrector):
     """Merges suggestions from a list of sub-correctors.
@@ -163,36 +144,39 @@ class MultiCorrector(Corrector):
     def __init__(self, correctors):
         self.correctors = correctors
 
-    def _suggestions(self, text, maxdist, prefix, seen):
+    def _suggestions(self, text, maxdist, prefix):
         for corr in self.correctors:
-            for item in corr._suggestions(text, maxdist, prefix, seen):
+            for item in corr._suggestions(text, maxdist, prefix):
                 yield item
 
 
-def wordlist_to_graph_file(wordlist, dbfile, strip=True):
+def wordlist_to_graph_file(wordlist, dbfile, fieldname="_", strip=True):
     """Writes a word graph file from a list of words.
-    
+
     >>> # Open a word list file with one word on each line, and write the
     >>> # word graph to a graph file
     >>> wordlist_to_graph_file("mywords.txt", "mywords.dawg")
-    
+
     :param wordlist: an iterable containing the words for the graph. The words
         must be in sorted order.
     :param dbfile: a filename string or file-like object to write the word
-        graph to. If you pass a file-like object, it will be closed when the
-        function completes.
+        graph to. This function will close the file.
     """
 
     from whoosh.filedb.structfile import StructFile
-
-    g = GraphCorrector.from_word_list(wordlist, strip=strip)
-
     if isinstance(dbfile, string_type):
         dbfile = open(dbfile, "wb")
     if not isinstance(dbfile, StructFile):
         dbfile = StructFile(dbfile)
 
-    g.to_file(dbfile)
+    gw = fst.GraphWriter(dbfile)
+    gw.start_field(fieldname)
+    for word in wordlist:
+        if strip:
+            word = word.strip()
+        gw.insert(word)
+    gw.finish_field()
+    gw.close()
 
 
 # Query correction
@@ -200,7 +184,7 @@ def wordlist_to_graph_file(wordlist, dbfile, strip=True):
 class Correction(object):
     """Represents the corrected version of a user query string. Has the
     following attributes:
-    
+
     ``query``
         The corrected :class:`whoosh.query.Query` object.
     ``string``
@@ -211,16 +195,16 @@ class Correction(object):
         The original user query string.
     ``tokens``
         A list of token objects representing the corrected words.
-    
-    You can also use the :meth:`Correction.format_string` to reformat the
+
+    You can also use the :meth:`Correction.format_string` method to reformat the
     corrected query string using a :class:`whoosh.highlight.Formatter` class.
     For example, to display the corrected query string as HTML with the
     changed words emphasized::
-    
+
         from whoosh import highlight
-        
+
         correction = mysearcher.correct_query(q, qstring)
-        
+
         hf = highlight.HtmlFormatter(classname="change")
         html = correction.format_string(hf)
     """
@@ -231,18 +215,26 @@ class Correction(object):
         self.original_string = qstring
         self.tokens = tokens
 
-        if self.original_string and self.tokens:
+        if self.original_string:
             self.string = self.format_string(highlight.NullFormatter())
         else:
-            self.string = None
+            self.string = ''
 
     def __repr__(self):
         return "%s(%r, %r)" % (self.__class__.__name__, self.query,
                                self.string)
 
     def format_string(self, formatter):
-        if not (self.original_string and self.tokens):
-            raise Exception("The original query isn't available")
+        """
+        Highlights the corrected words in the original query string using the
+        given :class:`~whoosh.highlight.Formatter`.
+
+        :param formatter: A :class:`whoosh.highlight.Formatter` instance.
+        :return: the output of the formatter (usually a string).
+        """
+
+        if not self.original_string:
+            return ''
         if isinstance(formatter, type):
             formatter = formatter()
 
@@ -259,7 +251,7 @@ class QueryCorrector(object):
     def correct_query(self, q, qstring):
         """Returns a :class:`Correction` object representing the corrected
         form of the given query.
-        
+
         :param q: the original :class:`whoosh.query.Query` tree to be
             corrected.
         :param qstring: the original user query. This may be None if the
@@ -306,145 +298,39 @@ class SimpleQueryCorrector(QueryCorrector):
         prefix = self.prefix
         maxdist = self.maxdist
 
+        # A list of tokens that were changed by a corrector
         corrected_tokens = []
+
+        # The corrected query tree. We don't need to deepcopy the original
+        # because we use Query.replace() to find-and-replace the corrected
+        # words and it returns a copy of the query tree.
         corrected_q = q
+
+        # For every word in the original query...
+        # Note we can't put these in a set, because we must preserve WHERE
+        # in the query each token occured so we can format them later
         for token in q.all_tokens():
             fname = token.fieldname
+
+            # If this is one of the words we're supposed to correct...
             if (fname, token.text) in termset:
                 sugs = correctors[fname].suggest(token.text, prefix=prefix,
                                                  maxdist=maxdist)
                 if sugs:
+                    # This is a "simple" corrector, so we just pick the first
+                    # suggestion :/
                     sug = sugs[0]
+
+                    # Return a new copy of the original query with this word
+                    # replaced by the correction
                     corrected_q = corrected_q.replace(token.fieldname,
                                                       token.text, sug)
+                    # Add the token to the list of corrected tokens (for the
+                    # formatter to use later)
+                    token.original = token.text
                     token.text = sug
                     corrected_tokens.append(token)
 
         return Correction(q, qstring, corrected_q, corrected_tokens)
 
 
-#
-#
-#
-#
-# Old, obsolete spell checker - DO NOT USE
-
-class SpellChecker(object):
-    """This feature is obsolete.
-    """
-
-    def __init__(self, storage, indexname="SPELL",
-                 booststart=2.0, boostend=1.0,
-                 mingram=3, maxgram=4,
-                 minscore=0.5):
-        self.storage = storage
-        self.indexname = indexname
-
-        self._index = None
-
-        self.booststart = booststart
-        self.boostend = boostend
-        self.mingram = mingram
-        self.maxgram = maxgram
-        self.minscore = minscore
-
-    def index(self, create=False):
-        from whoosh import index
-        if create or not self._index:
-            create = create or not index.exists(self.storage,
-                                                indexname=self.indexname)
-            if create:
-                self._index = self.storage.create_index(self._schema(),
-                                                        self.indexname)
-            else:
-                self._index = self.storage.open_index(self.indexname)
-        return self._index
-
-    def _schema(self):
-        # Creates a schema given this object's mingram and maxgram attributes.
-
-        from whoosh.fields import Schema, FieldType, ID, STORED
-        from whoosh.formats import Frequency
-        from whoosh.analysis import SimpleAnalyzer
-
-        idtype = ID()
-        freqtype = FieldType(Frequency(), SimpleAnalyzer())
-
-        fls = [("word", STORED), ("score", STORED)]
-        for size in xrange(self.mingram, self.maxgram + 1):
-            fls.extend([("start%s" % size, idtype),
-                        ("end%s" % size, idtype),
-                        ("gram%s" % size, freqtype)])
-
-        return Schema(**dict(fls))
-
-    def suggestions_and_scores(self, text, weighting=None):
-        if weighting is None:
-            weighting = scoring.TF_IDF()
-
-        grams = defaultdict(list)
-        for size in xrange(self.mingram, self.maxgram + 1):
-            key = "gram%s" % size
-            nga = analysis.NgramAnalyzer(size)
-            for t in nga(text):
-                grams[key].append(t.text)
-
-        queries = []
-        for size in xrange(self.mingram, min(self.maxgram + 1, len(text))):
-            key = "gram%s" % size
-            gramlist = grams[key]
-            queries.append(query.Term("start%s" % size, gramlist[0],
-                                      boost=self.booststart))
-            queries.append(query.Term("end%s" % size, gramlist[-1],
-                                      boost=self.boostend))
-            for gram in gramlist:
-                queries.append(query.Term(key, gram))
-
-        q = query.Or(queries)
-        ix = self.index()
-        s = ix.searcher(weighting=weighting)
-        try:
-            result = s.search(q, limit=None)
-            return [(fs["word"], fs["score"], result.score(i))
-                    for i, fs in enumerate(result)
-                    if fs["word"] != text]
-        finally:
-            s.close()
-
-    def suggest(self, text, number=3, usescores=False):
-        if usescores:
-            def keyfn(a):
-                return 0 - (1 / distance(text, a[0])) * a[1]
-        else:
-            def keyfn(a):
-                return distance(text, a[0])
-
-        suggestions = self.suggestions_and_scores(text)
-        suggestions.sort(key=keyfn)
-        return [word for word, _, weight in suggestions[:number]
-                if weight >= self.minscore]
-
-    def add_field(self, ix, fieldname):
-        r = ix.reader()
-        try:
-            self.add_scored_words((w, terminfo.weight())
-                                  for w, terminfo in r.iter_field(fieldname))
-        finally:
-            r.close()
-
-    def add_words(self, ws, score=1):
-        self.add_scored_words((w, score) for w in ws)
-
-    def add_scored_words(self, ws):
-        writer = self.index().writer()
-        for text, score in ws:
-            fields = {"word": text, "score": score}
-            for size in xrange(self.mingram, self.maxgram + 1):
-                nga = analysis.NgramAnalyzer(size)
-                gramlist = [t.text for t in nga(text)]
-                if len(gramlist) > 0:
-                    fields["start%s" % size] = gramlist[0]
-                    fields["end%s" % size] = gramlist[-1]
-                    fields["gram%s" % size] = " ".join(gramlist)
-            writer.add_document(**fields)
-        writer.commit()
